/* z_adjlist.c

   ZRAM: Adjacency lists, used by the connected induced subgraph enumeration

   Ambros Marzetta

   95-07-17
 * $Source: /home/schilte/marzetta/work/distr/zram/RCS/z_adjlist.c,v $
 * $Revision: 1.1 $
 * $Date: 1996/06/03 19:00:59 $
 * $Log: z_adjlist.c,v $
 * Revision 1.1  1996/06/03  19:00:59  marzetta
 * Initial revision
 *
 */

#include "z_adjlist.h"
#include <stdlib.h>		/* exit() */

z_adjlistP z_adjlist_New(int numPt, int numEdge)
{
  z_adjlistP n;

  n = (z_adjlistP) z_NewNode(z_adjlistTag,
			     sizeof(z_adjlistT) +
			     (2* numPt + numEdge) * sizeof(adjpointT));
  n->numPt = numPt;
  n->numEdge = numEdge;
  return n;
} /* z_adjlist_NewNode */

void z_adjlist_Free(z_adjlistP node)
{
  z_FreeNode((z_NodeP) node);
} /* z_adjlist_FreeNode */

void z_adjlist_ReadStart(z_adjlistP adjlist,
			 int *i,
			 int *index)
{
  *index = 0;			/* index into adjacency list */
  *i = 0;
} /* z_adjlist_ReadStart */

void z_adjlist_ReadEnd(z_adjlistP adjlist,
		       int *i,
		       int *index)
{
  if (*i != adjlist->numPt) {
    fprintf(stderr, "z_adjlist_Read: Read %d points instead of %d\n", *i,
	    adjlist->numPt);
    exit(1);
  }
  
  if (*index != adjlist->numPt + adjlist->numEdge) {
    fprintf(stderr, "z_adjlist_Read: Bad edge count (%d instead of"
	    "announced %d)\n",
	    *index - adjlist->numPt, adjlist->numEdge);
    if (*index > adjlist->numPt + adjlist->numEdge) exit(1);
    adjlist->numEdge = *index - adjlist->numPt;
  }
} /* z_adjlist_ReadEnd */

void z_adjlist_Read(FILE *infile,
		    z_adjlistP adjlist,
		    int *i,
		    int *index,
		    int maxdegree)
{
  int nr, matches;
  unsigned int degree, other;
  
  if (fscanf(infile, "%d:", &nr) != 1 || nr != *i) {
    fprintf(stderr, "z_adjlist_Read: Error with point %d (%d)\n", nr, *i);
    exit(1);		/* should deallocate */
  }
  degree = 0;
  adjlist_first(adjlist)[nr] = *index;
  
  for(;;) {			/* for (edge) */
    matches = fscanf(infile, "%d", &other);
    if (matches != 1) break;	/* breaks at 'P' and at end of file */
    degree++;
    if (degree > maxdegree) {
      fprintf(stderr, "z_adjlist_Read: degree of point %d is > %d\n",
	      nr, maxdegree);
      exit(1);		/* should deallocate */
    }
    if (other >= adjlist->numPt || other == nr) {
      if (other == nr)
	fprintf(stderr, "z_adjlist_Read: Unallowed loop (%d,%d)\n", nr, other);
      else
	fprintf(stderr, "z_adjlist_Read: Error with edge (%d,%d)"
		" (point not in graph)\n", nr, other);
      exit(1);		/* should deallocate */
    }
    if (other < nr) {		/* find inverse edge */
      adjpointT *adj2;
      adj2 = adjlist_adjPt(adjlist, other);
      while (*adj2 != nr && *adj2 != ADJ_LAST) adj2 ++;
      if (*adj2 == ADJ_LAST) {
	fprintf(stderr, "z_adjlist_Read: Edge (%d,%d) is not symmetric.\n",
		nr, other);
	exit(1);
      }
    } /* find inverse edge */
    adjlist_adj(adjlist)[(*index)++] = other;
  } /* for (edge) */
  adjlist_adj(adjlist)[(*index)++] = ADJ_LAST;
  (*i)++;
} /* z_adjlist_Read */

static boolean z_adjlist_inited;
int z_adjlistTag;

void z_adjlist_Init(void)
{
  if (! z_adjlist_inited) {
    z_adjlist_inited = 1;
    z_adjlistTag = z_InstallClass(z_adjlist, 0, 0, 0);
  }
} /* z_adjlist_Init */
