/* z_bb.h (ZRAM: Branch and Bound)
   Ambros Marzetta
   94-10-21

 * $Source: /home/schilte/marzetta/work/distr/zram/RCS/z_bb.h,v $
 * $Revision: 1.8 $
 * $Date: 1996/05/15 16:54:32 $
 * $Log: z_bb.h,v $
 * Revision 1.8  1996/05/15  16:54:32  marzetta
 * Added z_bb_RedistributeDFExp
 *
 * Revision 1.7  1996/05/08  20:28:09  marzetta
 * Added depth-first with explicit stack
 *
 * Revision 1.6  1996/04/09  13:15:08  marzetta
 * Estimator
 *
 * Revision 1.5  1995/07/31  14:43:48  marzetta
 * Branch in place
 *
 * Revision 1.4  1995/07/26  09:32:32  marzetta
 * Branch procedure gets current upper bound
 *
 * Revision 1.3  1995/07/25  11:53:43  marzetta
 * Added all my branch and bound procedures.
 *
 * Revision 1.2  1995/07/24  16:04:42  marzetta
 * Added noisy to depth first procedure, removed bb_free
 *
 * Revision 1.1  1995/02/24  13:57:16  marzetta
 * Initial revision
 *
 */

#ifndef _Z_BB_H_
#define _Z_BB_H_
#include "z_base.h"
#include "z_ps.h"

typedef z_NodeP z_bb_GlobalDataP;

typedef void (*z_bb_DoProcP)(z_NodeP np, void *ref);
typedef void (*z_bb_BranchProc)(z_bb_GlobalDataP global, z_NodeP np,
				z_NodeP upperBound, /* may be NULL */
				z_bb_DoProcP whatToDo, void *ref);
typedef z_NodeP (*z_bb_UpperBoundProc)(z_bb_GlobalDataP global, z_NodeP np);
typedef signed int (*z_bb_CompareProc)(z_NodeP x, z_NodeP y);
typedef boolean (*z_bb_IsSolutionProc)(z_NodeP np);
typedef char *(*z_bb_ShortNodeStringProc)(z_NodeP np);
typedef void (*z_bb_FreeProc)(z_NodeP np);

typedef struct z_bb_ProcSet {
  z_bb_BranchProc		bb_Branch; 		/* may be NULL */
  z_bb_BranchProc		bb_BranchInPlace; 	/* may be NULL */
  z_bb_UpperBoundProc		bb_UpperBound; 		/* may be NULL */
  z_bb_CompareProc		bb_CompareProc;
  z_bb_IsSolutionProc		bb_IsSolution;
  z_bb_ShortNodeStringProc	bb_ShortNodeString; 	/* may be NULL */
} z_bb_ProcSet, *z_bb_ProcSetP;

/* At least one of the two branch procedures must be different from NULL. */


/* File s_bb1.c --------------------------------------------------- */

z_NodeP z_BranchBoundDepthFirst(z_ProcSetNum procsetnum,
				z_bb_GlobalDataP global,
				z_NodeP root, z_NodeP initialUpperBound,
				int noisy);

z_NodeP z_BranchBoundBestFirst(z_ProcSetNum procsetnum,
			       z_bb_GlobalDataP global,
			       z_NodeP root, z_NodeP initialUpperBound,
			       int noisy);

z_NodeP z_BranchBoundMixedFirst(z_ProcSetNum procsetnum,
				z_bb_GlobalDataP global,
				z_NodeP root, z_NodeP initialUpperBound,
				int queueSize, int noisy);

void s_bb1_Init(void);


/* File z_bb1.c ---------------------------------------------------- */

#ifndef SEQUENTIAL

z_NodeP z_BranchBoundParallel(z_ProcSetNum procsetnum,
			      z_bb_GlobalDataP global,
			      z_NodeP root, z_NodeP initialUpperBound,
			      int queueSize, boolean noisy);

#endif /* SEQUENTIAL */

void z_bb1_Init(void);


/* File z_bb2.c ---------------------------------------------------- */

void z_BranchBoundEstimate(z_ProcSetNum procsetnum,
			   z_bb_GlobalDataP global,
			   z_NodeP root, z_NodeP UpperBound,
			   int numEstimates,
			   float *nNodes, float *sNodes,
			   float *nTime, float *sTime);
/* Estimate the number of nodes with a value less than UpperBound in the
   search tree. Evaluate numEstimates random paths through the tree.
   Return an estimate for the number of nodes and the (sequential) time
   together with the standard deviation of the estimate. */

void z_bb2_Init(void);


/* File z_bb3.c ---------------------------------------------------- */
/* Branch and Bound, depth first using an explicit stack, restartable */

z_NodeP z_BranchBoundDFExp(z_ProcSetNum procsetnum,
			   z_bb_GlobalDataP global,
			   z_NodeP root, z_NodeP initialUpperBound,
			   boolean noisy,
			   boolean sequential);

z_NodeP z_RestartBranchBoundDFExp(z_ProcSetNum procsetnum,
				  z_bb_GlobalDataP global,
				  boolean sequential);


void z_bb3_Init(void);


/* File z_bb4.c ---------------------------------------------------- */
/* Redistribute z_BranchBoundDFExp checkpoints */

int z_bb_RedistributeDFExp(z_ProcSetNum procsetnum, int n_in, int n_out);
/* Read checkpoint files created by n_in processors and write them as
   if they were created by n_out processors. Print the statistical data
   and reinitialize it to zero.
   Return zero if there is no error.
   */

void z_bb4_Init(void);

#endif /* _Z_BB_H_ */
