/* z_bt2.c (ZRAM: Backtracking (sequential depth first)
   Ambros Marzetta

 * $Source: /home/schilte/marzetta/work/distr/zram/RCS/z_bt2.c,v $
 * $Revision: 1.1 $
 * $Date: 1998/03/10 15:03:35 $
 * $Log: z_bt2.c,v $
 * Revision 1.1  1998/03/10  15:03:35  marzetta
 * Initial revision
 *
 */

#include "z_bt.h"
#include "z_lb.h"
#include "z_mp.h"
#include <stdlib.h>

static boolean bt2_inited;

/* All procedures stop and return the current node when the branching
   procedure returns true. They return NULL if the branching procedure
   always returns false.
   */

/* Variables ----------------------------------------------------------- */

static struct {
  z_bt_GlobalP app_global;
  z_bt_BranchProc BranchInPlace;
  s_seqP workList;
  z_NodeP solution;
} glob;

/* Parallel / Sequential using z_lb.h ------------------------------------ */

static z_ProcSetNum bt_procsetnum;
static z_lb_ProcSet bt_procset;
static int bt2_GlobalTag;

typedef struct bt2_Global {
  Z_NODE_HEADER;		/* len; tag = bt2_GlobalTag */
  z_NodeP bt_app_global;	/* application global node */
  int bt_app_psn;		/* application procedure set (z_bt_ProcSetP) */
} bt2_Global, *bt2_GlobalP;

static bt2_GlobalP NewBtGlobal(z_ProcSetNum procsetnum, z_lb_GlobalP ap_global)
{
  bt2_GlobalP bt2_g;

  assert(ap_global);
  bt2_g = (bt2_GlobalP) z_NewNode(bt2_GlobalTag, sizeof(bt2_Global));
  bt2_g->bt_app_psn = procsetnum;
  bt2_g->bt_app_global = ap_global;
  return bt2_g;
} /* NewBtGlobal */

void z_bt2_lb_Do(z_NodeP child, void *ref)
{
  /* This procedure is called for all children of the node just
     extracted from the sequence. The child goes to
     the end of the sequence. (depth-first)
     */
  assert(child);

  s_seq_InsertLast(glob.workList, z_NewCopyNode(child));
} /* z_bt2_lb_Do */

void bt2_WorkAndSplit(bt2_GlobalP global,
		     z_lb_WorkUnitP workUnit,
		     s_seqP workList)
     /* called from z_lb through bt_procset */
{
  z_bt_ProcSetP psp;

  psp = (z_bt_ProcSetP) z_GetProcSet(global->bt_app_psn);
  glob.BranchInPlace = psp->bt_BranchInPlace;
  glob.workList = workList;
  glob.app_global = global->bt_app_global;
  glob.solution = NULL;
  glob.BranchInPlace(glob.app_global, workUnit, &z_bt2_lb_Do, NULL);
  z_FreeNode(workUnit);
} /* bt2_WorkAndSplit */

z_NodeP z_bt_EnumerateDFExp(z_ProcSetNum procsetnum,
		       z_bt_GlobalP global,
		       z_NodeP root,
		       boolean sequential)
{
  bt2_GlobalP bt2_Global;
  z_bt_ProcSetP psp;

  assert(bt2_inited);
  assert(z_frontend);
  psp = (z_bt_ProcSetP) z_GetProcSet(procsetnum);
  assert(psp->bt_BranchInPlace);

  bt2_Global = NewBtGlobal(procsetnum, global);
  z_lb_Work(bt_procsetnum, bt2_Global, root, sequential);
  return NULL;
} /* z_bt_EnumerateDFExp */


/* Initialisation ------------------------------------------------------- */

void z_bt2_Init(void)
{
  /* Initialize this module */
  if (! bt2_inited) {
    bt2_inited = 1;
    bt2_GlobalTag = z_InstallClass(bt2_Global, 1, 0, 0);
    z_lb_Init();
    bt_procset.lb_Work = NULL;
    bt_procset.lb_Split = NULL;
    bt_procset.lb_WorkAndSplit = (z_lb_SplitProc) bt2_WorkAndSplit;
    bt_procset.lb_Fini = NULL;
    bt_procsetnum = z_InstallProcSet((z_ProcSetP) &bt_procset);
  }
} /* z_bt2_Init */

