/* z_main.c

   Main program which initializes the parallel computer, calls
   initproc() on every processor and calls mmain() on the front end.
   
   Not compiled for PEonly.

   Ambros Marzetta
   95-04-21

 * $Source: /home/schilte/marzetta/work/distr/zram/RCS/z_main.c,v $
 * $Revision: 1.10 $
 * $Date: 1996/04/09 13:21:58 $
 * $Log: z_main.c,v $
 * Revision 1.10  1996/04/09  13:21:58  marzetta
 * Added SEQUENTIAL
 *
 * Revision 1.9  1995/08/22  12:39:45  marzetta
 * New message passing, MPI
 *
 * Revision 1.7  1995/08/03  16:09:38  bruengg
 * improved reading arguments for the mac
 *
 * Revision 1.5  1995/08/03  08:56:13  marzetta
 * Macintosh portability
 *
 * Revision 1.4  1995/07/28  14:05:58  marzetta
 * Some cosmetic changes
 *
 */

#include <stdio.h>
#include <stdlib.h>		/* exit() */
#include "z_par.h"
#include "z_par_intern.h"
#include "z_md.h"		/* z_md_Init() */

#if defined(macintosh)
#include <ctype.h>

#define maxChars 100
#define maxArgs   10

static char commandline[100] = "";
static char *macargv[10];

static void MacReadArgv(int *argc, char ***argv)
/* (*argc)[0] is the empty string on the mac since there's no command name */
{
  int nrChars;
  char ch;
    
  /* allocating memory for the argv */
  *argv = calloc(maxArgs, sizeof(&commandline));
  
  printf("Arguments : ");
  nrChars = 0;
  *argc = 1;
  (*argv[0]) = commandline;
  do
	{
		/* overread white space */
		do
			ch = (char) getc(stdin);
		while (isspace(ch) && !(ch == '\n'));
		/* read next argument */
 		if (ch != '\n')
 			{
				commandline[nrChars] = '\0';
				nrChars++;
 				assert(nrChars < maxChars);	
 		 		assert(nrChars < maxChars);
 				(*argv)[*argc] = commandline + nrChars;
 				*argc = *argc + 1;
 			}
		while (!isspace(ch) && !(ch == '\n'))
			{
				commandline[nrChars] = ch;
	  			nrChars++;
 				assert(nrChars < maxChars);	
	  			ch = (char) getc(stdin);
			}
	}
  while (ch != '\n');
  commandline[nrChars] = '\0';
} /* MacReadArgv */

#endif /* macintosh */

static int main_fe(int argc, char **argv, char **envp)
{
  int retval;

  if (0)
    if (z_numprocs > 1)
      printf("-- %d processors --\r\n", z_numprocs);
    else
      printf("-- 1 processor --\n");
  retval =  mmain(argc, argv, envp); /* application dependent */
    
  return retval;
} /* main_fe */

int main(int argc, char **argv, char **envp)
{
  int result;
#if defined(macintosh)
  MacReadArgv(&argc, &argv);
  envp = NULL;
#endif /* macintosh */
  z_md_Init(&argc, &argv);		/* machine dependent initialisation */
  initproc();			/* application dependent initialisation */
#ifdef SEQUENTIAL
  assert(z_frontend);
  result = main_fe(argc, argv, envp);
#else /* SEQUENTIAL */
  if (z_frontend)
    result = main_fe(argc, argv, envp);
  else
    z_pe_loop(), result = 0;
#endif /* SEQUENTIAL */
  z_par_Fin();
  return result;
} /* main */
